# -*- coding: utf-8 -*-

import sys
import logging

from django.core.context_processors import csrf
from django.contrib.auth.views import login, logout
from django.contrib import auth
from django.http import HttpResponseRedirect
from django.shortcuts import render_to_response

import backend.sharedvariables as var
from backend.dbinterface import get_installations, rename_agent, delete_agent, \
                    get_installation_log, new_installation, get_agent_name
from backend.servertoagent import installation_request, exit_request
from backend.server import start_server
from backend.recipes import *
from backend.servers import *
from django.contrib.auth.decorators import login_required


def setup_logger(log_file, rotation=5*1024, count=5):
    """ Настраивает логирование в STDOUT и указанный лог-файл @a log_file.
        Объём ротации задаётся 
    """
    formatter = logging.Formatter('[%(asctime)s][%(levelname)s][%(module)s]: %(message)s', datefmt='%H:%M:%S %d-%m-%Y')
    
    handler = logging.handlers.RotatingFileHandler(log_file, maxBytes=rotation*1024, backupCount=count)
    handler.setFormatter(formatter)
    logging.getLogger().addHandler(handler)
    
    console = logging.StreamHandler()
    console.setFormatter(formatter)
    logging.getLogger().addHandler(console)
    
    logging.getLogger().setLevel(logging.DEBUG)

def server_run():           
    if var.server_state != 'started': 
        try:
            setup_logger('/var/log/junta-webserver.log')
        except IOError:
            print >>sys.stderr, 'Could not setup logging to file'
        logging.info('starting...')
        start_server()
        var.server_state = 'started'
        logging.info('started.')


def get_common_context (request):
    try:
        server_run()
        c = {}
        c.update (csrf(request))
        c['user'] = request.user
        c['recipes'] = get_recipes()
        c['servers'] = get_servers()
        c['server_is_started'] = (var.server_state == 'started')
        return c
    except Exception as e:
        logging.error(e)

def get_server_context(server_id):
    try:
        c = {}
        c['server_id'] = server_id
        c['server_name'] = get_agent_name(server_id)
        c['status'] = get_agent_status(server_id)
        c['installations'] = get_installations(server_id)
        return c
    except Exception as e:
        logging.error(e)

@login_required
def home(request):
    try:
        c = get_common_context(request)
        return render_to_response ('home.html', c)
    except Exception as e:
        logging.error(e)
        return HttpResponseRedirect ("/")
    
@login_required
def recipes(request, recipe_name, recipe_version):
    try:
        c = get_common_context(request)
        if request.method == 'POST':
            raw_content = request.POST["content"]
            file_content = raw_content.encode('utf-8')
            err_msg = not_valid_error_message(file_content) 
            if err_msg is None:
                recipe_name = request.POST['recipe_name']
                recipe_version = get_next_version(recipe_name, c['recipes'])
                create_recipe_path_if_not_exists(recipe_name)
                err_msg = recipe_save(recipe_name, recipe_version, file_content, request.user.username)
                if err_msg is None:# OK
                    return HttpResponseRedirect('/recipes/' + recipe_name + '/' + recipe_version)
            c['err_msg'] = err_msg
        else:
            if recipe_name == "add_recipe":
                recipe_name = "new_recipe"
                recipe_version = '1'
                try:
                    file_content = get_recipe_template()
                except:
                    file_content = ''
            else:
                file_content = get_recipe_content(recipe_name, recipe_version)
        c['recipe_name'] = recipe_name
        c['recipe_version'] = recipe_version
        c['file_content'] = file_content.decode('utf-8')
        return render_to_response ('recipes.html', c)
    except Exception as e:
        logging.error(e)
        return HttpResponseRedirect ("/")

def installation_begin(c, request, server_id):
    try:
        recipe_full = request.POST['recipe'].split('/')
        c['recipe_name'] = recipe_full[0]
        c['recipe_version'] = recipe_full[1]
        c['config'] = get_default_config(c['recipe_name'], c['recipe_version'])
        c['is_installing'] = True
        c['getting_config'] = True
        c.update(get_server_context(server_id))
        return render_to_response ('installation.html', c)
    except Exception as e:
        logging.error(e)
        return HttpResponseRedirect ("/")

def installation_ready(c, request, server_id):
    try:
        recipe_name = request.POST['recipe_name']
        recipe_version = request.POST['recipe_version']
        c.update(get_server_context(server_id))
        config = {}
        for arg in request.POST:
            if '.' in arg:
                key = arg.split('.')[1]
                val = request.POST[arg]
                config[key] = val
        default_config = get_default_config(recipe_name, recipe_version)
        print " default_config: ", default_config
        hidden_config_keys = []
        for d_c in default_config:
            if d_c[1] == '':
                config[d_c[0]] = d_c[1]
                print " * ", d_c
                if d_c[2] == 'password':
                    hidden_config_keys.append(d_c[0])
                    
        to_agent = agents_list[server_id]
        if to_agent is not None:
            print '\nhidden_config: ', hidden_config_keys
            new_installation(server_id, recipe_name, recipe_version, config, hidden_config_keys)
            to_agent.reset_counter()
            to_agent.write_message(installation_request(recipe_name, recipe_version, config))
            return HttpResponseRedirect("/".join(["/installation", server_id, recipe_name, recipe_version]))
        else:
            return HttpResponseRedirect('/')
    except Exception as e:
        logging.error(e)
        return HttpResponseRedirect ("/")

def installation(c, request, server_id):
    try:
        c['is_installing'] = True
        if request.POST['action'] == u'Установить':
            return installation_begin(c, request, server_id)
        elif request.POST['action'] == u'Готово':
            return installation_ready(c, request, server_id)
        else:   # "Cancel" pressed
            if "server_name" in request.POST:
                return HttpResponseRedirect("/servers/" + server_id)
            else:
                return HttpResponseRedirect('/')
    except Exception as e:
        logging.error(e)
        return HttpResponseRedirect ("/")

@login_required
def installation_log(request, server_id, recipe_name, recipe_version):
    try:
        c = get_common_context(request)
        c['log'] = get_installation_log(server_id, recipe_name, recipe_version)   
        c['is_installing'] = True
        c.update(get_server_context(server_id))
        return render_to_response ('installation.html', c)
    except Exception as e:
        logging.error(e)
        return HttpResponseRedirect ("/")

@login_required
def servers(request, server_id):
    try:
        c = get_common_context(request)
        if request.method == 'POST':
            return installation(c, request, server_id)
        else:
            c.update(get_server_context(server_id))
            return render_to_response ('servers.html', c)
    except Exception as e:
        logging.error(e)
        return HttpResponseRedirect ("/")

@login_required
def rename(request, server_id):
    try:
        if request.method == 'POST':
            if request.POST["action"] == u'Сохранить':
                rename_agent(request.POST["id"], request.POST["new_name"])
            return HttpResponseRedirect('/servers/' + request.POST["id"])
        else:
            c = get_common_context(request)
            c.update(get_server_context(server_id))
            c['rename'] = 'Yes'
            return render_to_response ('servers.html', c)
    except Exception as e:
        logging.error(e)
        return HttpResponseRedirect ("/")

@login_required
def delete(request, server_id):
    try:
        delete_agent(server_id)
        to_agent = agents_list[server_id]
        if to_agent is not None:
            to_agent.write_message(exit_request())
        return HttpResponseRedirect('/')
    except Exception as e:
        logging.error(e)
        return HttpResponseRedirect ("/")

def login_user(request, params=''):
    try:
        c = {}
        c.update(csrf(request))
        if request.method == 'POST':
            username = request.POST.get('username', '')
            password = request.POST.get('password', '')
            user = auth.authenticate(username = username, password = password)
            if user is not None:
                if user.is_active:
                    login(request, user)
                    return HttpResponseRedirect("/")
            c['err_message'] = 'Неправильный логин и/или пароль'
            return render_to_response('auth.html', c)
        else:
            return render_to_response('auth.html', c)
    except Exception as e:
        logging.error(e)
        return HttpResponseRedirect ("/")
    
def logout_user (request):
    try:
        logout(request)
        return HttpResponseRedirect ("/auth")
    except Exception as e:
        logging.error(e)
        return HttpResponseRedirect ("/")